@@ -46,11 +46,11 @@ rm_conffile() {
 	[ -n "$1" ] || error "maintainer script parameters are missing"
 	[ -n "$DPKG_MAINTSCRIPT_NAME" ] || \
 		error "environment variable DPKG_MAINTSCRIPT_NAME is required"
+	[ -n "$DPKG_MAINTSCRIPT_PACKAGE" ] || \
+		error "environment variable DPKG_MAINTSCRIPT_PACKAGE is required"
 	[ "${CONFFILE}" != "${CONFFILE#/}" ] || \
 		error "conffile '$CONFFILE' is not an absolute path"
-	# Use --compare-versions to validate the version number.
-	[ -z "$(dpkg --compare-versions -- "$LASTVERSION" eq '0' 2>&1)" ] || \
-		error "version '$LASTVERSION' is not valid"
+	validate_optional_version "$LASTVERSION"
 
 	debug "Executing $0 rm_conffile in $DPKG_MAINTSCRIPT_NAME" \
 	      "of $DPKG_MAINTSCRIPT_PACKAGE"
@@ -96,7 +96,7 @@ prepare_rm_conffile() {
 	local md5sum old_md5sum
 	md5sum="$(md5sum "$CONFFILE" | sed -e 's/ .*//')"
 	old_md5sum="$(dpkg-query -W -f='${Conffiles}' "$PACKAGE" | \
-		sed -n -e "\'^ $CONFFILE ' { s/ obsolete$//; s/.* //; p }")"
+		sed -n -e "\\'^ $CONFFILE ' { s/ obsolete$//; s/.* //; p }")"
 	if [ "$md5sum" != "$old_md5sum" ]; then
 		mv -f "$CONFFILE" "$CONFFILE.dpkg-backup"
 	else
@@ -129,7 +129,7 @@ abort_rm_conffile() {
 		mv "$CONFFILE.dpkg-remove" "$CONFFILE"
 	fi
 	if [ -e "$CONFFILE.dpkg-backup" ]; then
-		echo "Reinstalling $CONFFILE that was backupped"
+		echo "Reinstalling $CONFFILE that was backed-up"
 		mv "$CONFFILE.dpkg-backup" "$CONFFILE"
 	fi
 }
@@ -158,13 +158,13 @@ mv_conffile() {
 	[ -n "$1" ] || error "maintainer script parameters are missing"
 	[ -n "$DPKG_MAINTSCRIPT_NAME" ] || \
 		error "environment variable DPKG_MAINTSCRIPT_NAME is required"
+	[ -n "$DPKG_MAINTSCRIPT_PACKAGE" ] || \
+		error "environment variable DPKG_MAINTSCRIPT_PACKAGE is required"
 	[ "${OLDCONFFILE}" != "${OLDCONFFILE#/}" ] || \
 		error "old-conffile '$OLDCONFFILE' is not an absolute path"
 	[ "${NEWCONFFILE}" != "${NEWCONFFILE#/}" ] || \
 		error "new-conffile '$NEWCONFFILE' is not an absolute path"
-	# Use --compare-versions to validate the version number.
-	[ -z "$(dpkg --compare-versions -- "$LASTVERSION" eq '0' 2>&1)" ] || \
-		error "version '$LASTVERSION' is not valid"
+	validate_optional_version "$LASTVERSION"
 
 	debug "Executing $0 mv_conffile in $DPKG_MAINTSCRIPT_NAME" \
 	      "of $DPKG_MAINTSCRIPT_PACKAGE"
@@ -207,7 +207,7 @@ prepare_mv_conffile() {
 	local md5sum old_md5sum
 	md5sum="$(md5sum "$CONFFILE" | sed -e 's/ .*//')"
 	old_md5sum="$(dpkg-query -W -f='${Conffiles}' "$PACKAGE" | \
-		sed -n -e "\'^ $CONFFILE ' { s/ obsolete$//; s/.* //; p }")"
+		sed -n -e "\\'^ $CONFFILE ' { s/ obsolete$//; s/.* //; p }")"
 	if [ "$md5sum" = "$old_md5sum" ]; then
 		mv -f "$CONFFILE" "$CONFFILE.dpkg-remove"
 	fi
@@ -266,6 +266,8 @@ symlink_to_dir() {
 
 	[ -n "$DPKG_MAINTSCRIPT_NAME" ] || \
 		error "environment variable DPKG_MAINTSCRIPT_NAME is required"
+	[ -n "$DPKG_MAINTSCRIPT_PACKAGE" ] || \
+		error "environment variable DPKG_MAINTSCRIPT_PACKAGE is required"
 	[ -n "$PACKAGE" ] || error "cannot identify the package"
 	[ -n "$SYMLINK" ] || error "symlink parameter is missing"
 	[ "${SYMLINK#/}" = "$SYMLINK" ] && \
@@ -274,6 +276,7 @@ symlink_to_dir() {
 		error "symlink pathname ends with a slash"
 	[ -n "$SYMLINK_TARGET" ] || error "original symlink target is missing"
 	[ -n "$1" ] || error "maintainer script parameters are missing"
+	validate_optional_version "$LASTVERSION"
 
 	debug "Executing $0 symlink_to_dir in $DPKG_MAINTSCRIPT_NAME" \
 	      "of $DPKG_MAINTSCRIPT_PACKAGE"
@@ -343,12 +346,15 @@ dir_to_symlink() {
 
 	[ -n "$DPKG_MAINTSCRIPT_NAME" ] || \
 		error "environment variable DPKG_MAINTSCRIPT_NAME is required"
+	[ -n "$DPKG_MAINTSCRIPT_PACKAGE" ] || \
+		error "environment variable DPKG_MAINTSCRIPT_PACKAGE is required"
 	[ -n "$PACKAGE" ] || error "cannot identify the package"
 	[ -n "$PATHNAME" ] || error "directory parameter is missing"
 	[ "${PATHNAME#/}" = "$PATHNAME" ] && \
 		error "directory parameter is not an absolute path"
 	[ -n "$SYMLINK_TARGET" ] || error "new symlink target is missing"
 	[ -n "$1" ] || error "maintainer script parameters are missing"
+	validate_optional_version "$LASTVERSION"
 
 	debug "Executing $0 dir_to_symlink in $DPKG_MAINTSCRIPT_NAME" \
 	      "of $DPKG_MAINTSCRIPT_PACKAGE"
@@ -416,16 +422,12 @@ prepare_dir_to_symlink()
 
 	# If there are locally created files or files owned by another package
 	# we should not perform the switch.
-	find "$PATHNAME" -print0 | xargs -0 -n1 sh -c '
-		package="$1"
-		file="$2"
-		if ! dpkg-query -L "$package" | grep -F -q -x "$file"; then
-			exit 1
-		fi
-		exit 0
-	' check-files-ownership "$PACKAGE" || \
+	export DPKG_MAINTSCRIPT_HELPER_INTERNAL_API="$version"
+	find "$PATHNAME" -print0 | \
+		xargs -0 -n1 "$0" _internal_pkg_must_own_file "$PACKAGE" || \
 		error "directory '$PATHNAME' contains files not owned by" \
 		      "package $PACKAGE, cannot switch to symlink"
+	unset DPKG_MAINTSCRIPT_HELPER_INTERNAL_API
 
 	# At this point, we know that the directory either contains no files,
 	# or only non-conffiles owned by the package.
@@ -495,6 +497,18 @@ abort_dir_to_symlink()
 }
 
 # Common functions
+validate_optional_version() {
+	local VERSION="$1"
+
+	if [ -z "$VERSION" ]; then
+		return
+	fi
+
+	if ! VERSIONCHECK=$(dpkg --validate-version -- "$VERSION" 2>&1); then
+		error "$VERSIONCHECK"
+	fi
+}
+
 ensure_package_owns_file() {
 	local PACKAGE="$1"
 	local FILE="$2"
@@ -507,6 +521,21 @@ ensure_package_owns_file() {
 	return 0
 }
 
+internal_pkg_must_own_file()
+{
+	local PACKAGE="$1"
+	local FILE="$2"
+
+	if [ "$DPKG_MAINTSCRIPT_HELPER_INTERNAL_API" != "$version" ]; then
+		error "internal API used by external command"
+	fi
+
+	if ! ensure_package_owns_file "$PACKAGE" "$FILE"; then
+		error "file '$FILE' not owned by package '$PACKAGE'"
+	fi
+	return 0
+}
+
 symlink_match()
 {
 	local SYMLINK="$1"
@@ -516,19 +545,72 @@ symlink_match()
 	[ "$(readlink -f "$SYMLINK")" = "$SYMLINK_TARGET" ]
 }
 
+# Standard ANSI colors and attributes.
+COLOR_NORMAL=''
+COLOR_RESET='[0m'
+COLOR_BOLD='[1m'
+COLOR_BLACK='[30m'
+COLOR_RED='[31m'
+COLOR_GREEN='[32m'
+COLOR_YELLOW='[33m'
+COLOR_BLUE='[34m'
+COLOR_MAGENTA='[35m'
+COLOR_CYAN='[36m'
+COLOR_WHITE='[37m'
+COLOR_BOLD_BLACK='[1;30m'
+COLOR_BOLD_RED='[1;31m'
+COLOR_BOLD_GREEN='[1;32m'
+COLOR_BOLD_YELLOW='[1;33m'
+COLOR_BOLD_BLUE='[1;34m'
+COLOR_BOLD_MAGENTA='[1;35m'
+COLOR_BOLD_CYAN='[1;36m'
+COLOR_BOLD_WHITE='[1;37m'
+
+setup_colors()
+{
+	: "${DPKG_COLORS=auto}"
+
+	case "$DPKG_COLORS" in
+	auto)
+		if [ -t 1 ]; then
+			USE_COLORS=yes
+		else
+			USE_COLORS=no
+		fi
+		;;
+	always)
+		USE_COLORS=yes
+		;;
+	*)
+		USE_COLORS=no
+		;;
+	esac
+
+	if [ $USE_COLORS = yes ]; then
+		COLOR_PROG="$COLOR_BOLD"
+		COLOR_INFO="$COLOR_GREEN"
+		COLOR_NOTICE="$COLOR_YELLOW"
+		COLOR_WARN="$COLOR_BOLD_YELLOW"
+		COLOR_ERROR="$COLOR_BOLD_RED"
+	else
+		COLOR_RESET=""
+	fi
+	FMT_PROG="$COLOR_PROG$PROGNAME$COLOR_RESET"
+}
+
 debug() {
 	if [ -n "$DPKG_DEBUG" ]; then
-		echo "DEBUG: $PROGNAME: $*" >&2
+		echo "DEBUG: $FMT_PROG: $*" >&2
 	fi
 }
 
 error() {
-	echo "$PROGNAME: error: $*" >&2
+	echo "$FMT_PROG: ${COLOR_ERROR}error${COLOR_RESET}: $*" >&2
 	exit 1
 }
 
 warning() {
-	echo "$PROGNAME: warning: $*" >&2
+	echo "$FMT_PROG: ${COLOR_WARN}warning${COLOR_RESET}: $*" >&2
 }
 
 usage() {
@@ -555,7 +637,7 @@ END
 }
 
 badusage() {
-	echo "$PROGNAME: error: $1" >&2
+	echo "$FMT_PROG: ${COLOR_ERROR}error${COLOR_RESET}: $1" >&2
 	echo >&2
 	echo "Use '$PROGNAME help' for program usage information." >&2
 	exit 1
@@ -566,6 +648,9 @@ set -e
 
 PROGNAME=$(basename "$0")
 version="unknown"
+
+setup_colors
+
 command="$1"
 [ $# -gt 0 ] || badusage "missing command"
 shift
@@ -602,6 +687,10 @@ symlink_to_dir)
 dir_to_symlink)
 	dir_to_symlink "$@"
 	;;
+_internal_pkg_must_own_file)
+	# This is an internal command, must not be used outside this program.
+	internal_pkg_must_own_file "$@"
+	;;
 --help|help|-?)
 	usage
 	;;
