@@ -29,7 +29,6 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <string.h>
 #include <time.h>
@@ -56,12 +55,12 @@
 #include <dpkg/tarfn.h>
 #include <dpkg/options.h>
 #include <dpkg/triglib.h>
+#include <dpkg/db-ctrl.h>
+#include <dpkg/db-fsys.h>
 
-#include "filesdb.h"
 #include "main.h"
 #include "archives.h"
 #include "filters.h"
-#include "infodb.h"
 
 static inline void
 fd_writeback_init(int fd)
@@ -114,11 +113,11 @@ tar_pool_release(void)
   }
 }
 
-struct fileinlist *
-tar_filenamenode_queue_push(struct filenamenode_queue *queue,
-                            struct filenamenode *namenode)
+struct fsys_namenode_list *
+tar_fsys_namenode_queue_push(struct fsys_namenode_queue *queue,
+                            struct fsys_namenode *namenode)
 {
-  struct fileinlist *node;
+  struct fsys_namenode_list *node;
 
   node = tar_pool_alloc(sizeof(*node));
   node->namenode = namenode;
@@ -131,9 +130,9 @@ tar_filenamenode_queue_push(struct filenamenode_queue *queue,
 }
 
 static void
-tar_filenamenode_queue_pop(struct filenamenode_queue *queue,
-                           struct fileinlist **tail_prev,
-                           struct fileinlist *node)
+tar_fsys_namenode_queue_pop(struct fsys_namenode_queue *queue,
+                           struct fsys_namenode_list **tail_prev,
+                           struct fsys_namenode_list *node)
 {
   tar_pool_free(node);
   queue->tail = tail_prev;
@@ -149,11 +148,11 @@ tar_filenamenode_queue_pop(struct filenamenode_queue *queue,
  * shared directories don't stop packages from disappearing).
  */
 bool
-filesavespackage(struct fileinlist *file,
+filesavespackage(struct fsys_namenode_list *file,
                  struct pkginfo *pkgtobesaved,
                  struct pkginfo *pkgbeinginstalled)
 {
-  struct filepackages_iterator *iter;
+  struct fsys_node_pkgs_iter *iter;
   struct pkgset *divpkgset;
   struct pkginfo *thirdpkg;
 
@@ -172,14 +171,14 @@ filesavespackage(struct fileinlist *file,
     }
   }
   /* Is the file in the package being installed? If so then it can't save. */
-  if (file->namenode->flags & fnnf_new_inarchive) {
+  if (file->namenode->flags & FNNF_NEW_INARCHIVE) {
     debug(dbg_eachfiledetail,"filesavespackage ... in new archive -- no save");
     return false;
   }
   /* Look for a 3rd package which can take over the file (in case
    * it's a directory which is shared by many packages. */
-  iter = filepackages_iter_new(file->namenode);
-  while ((thirdpkg = filepackages_iter_next(iter))) {
+  iter = fsys_node_pkgs_iter_new(file->namenode);
+  while ((thirdpkg = fsys_node_pkgs_iter_next(iter))) {
     debug(dbg_eachfiledetail, "filesavespackage ... also in %s",
           pkg_name(thirdpkg, pnaw_always));
 
@@ -199,17 +198,17 @@ filesavespackage(struct fileinlist *file,
      * we shouldn't try to make it take over this shared directory. */
     debug(dbg_eachfiledetail,"filesavespackage ...  is 3rd package");
 
-    if (!thirdpkg->clientdata->fileslistvalid) {
+    if (!thirdpkg->files_list_valid) {
       debug(dbg_eachfiledetail, "process_archive ... already disappeared!");
       continue;
     }
 
     /* We've found a package that can take this file. */
     debug(dbg_eachfiledetail, "filesavespackage ...  taken -- no save");
-    filepackages_iter_free(iter);
+    fsys_node_pkgs_iter_free(iter);
     return false;
   }
-  filepackages_iter_free(iter);
+  fsys_node_pkgs_iter_free(iter);
 
   debug(dbg_eachfiledetail, "filesavespackage ... not taken -- save !");
   return true;
@@ -217,7 +216,7 @@ filesavespackage(struct fileinlist *file,
 
 static void
 md5hash_prev_conffile(struct pkginfo *pkg, char *oldhash, const char *oldname,
-                      struct filenamenode *namenode)
+                      struct fsys_namenode *namenode)
 {
   struct pkginfo *otherpkg;
   struct conffile *conff;
@@ -228,14 +227,10 @@ md5hash_prev_conffile(struct pkginfo *pkg, char *oldhash, const char *oldname,
   for (otherpkg = &pkg->set->pkg; otherpkg; otherpkg = otherpkg->arch_next) {
     if (otherpkg == pkg)
       continue;
-    /* The hash in the Conffiles is only meaningful if the package
-     * configuration has been at least tried. */
-    if (otherpkg->status < PKG_STAT_UNPACKED)
-      continue;
     /* If we are reinstalling, even if the other package is only unpacked,
      * we can always make use of the Conffiles hash value from an initial
      * installation, if that happened at all. */
-    if (otherpkg->status == PKG_STAT_UNPACKED &&
+    if (otherpkg->status <= PKG_STAT_UNPACKED &&
         dpkg_version_compare(&otherpkg->installed.version,
                              &otherpkg->configversion) != 0)
       continue;
@@ -269,8 +264,10 @@ void cu_pathname(int argc, void **argv) {
   path_remove_tree((char*)(argv[0]));
 }
 
-int tarfileread(void *ud, char *buf, int len) {
-  struct tarcontext *tc= (struct tarcontext*)ud;
+int
+tarfileread(struct tar_archive *tar, char *buf, int len)
+{
+  struct tarcontext *tc = (struct tarcontext *)tar->ctx;
   int r;
 
   r = fd_read(tc->backendpipe, buf, len);
@@ -348,16 +345,17 @@ does_replace(struct pkginfo *new_pkg, struct pkgbin *new_pkgbin,
 static void
 tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
                   const char *path, struct file_stat *st,
-                  struct filenamenode *namenode)
+                  struct fsys_namenode *namenode)
 {
   static struct varbuf hardlinkfn;
   static int fd;
 
   struct dpkg_error err;
-  struct filenamenode *linknode;
+  struct fsys_namenode *linknode;
   char fnamebuf[256];
   char fnamenewbuf[256];
   char *newhash;
+  int rc;
 
   switch (te->type) {
   case TAR_FILETYPE_FILE:
@@ -367,7 +365,7 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
     if (fd < 0)
       ohshite(_("unable to create '%.255s' (while processing '%.255s')"),
               path, te->name);
-    push_cleanup(cu_closefd, ehflag_bombout, NULL, 0, 1, &fd);
+    push_cleanup(cu_closefd, ehflag_bombout, 1, &fd);
     debug(dbg_eachfiledetail, "tarobject file open size=%jd",
           (intmax_t)te->size);
 
@@ -393,14 +391,16 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
             namenode->statoverride->uid,
             namenode->statoverride->gid,
             namenode->statoverride->mode);
-    if (fchown(fd, st->uid, st->gid))
+    rc = fchown(fd, st->uid, st->gid);
+    if (forcible_nonroot_error(rc))
       ohshite(_("error setting ownership of '%.255s'"), te->name);
-    if (fchmod(fd, st->mode & ~S_IFMT))
+    rc = fchmod(fd, st->mode & ~S_IFMT);
+    if (forcible_nonroot_error(rc))
       ohshite(_("error setting permissions of '%.255s'"), te->name);
 
     /* Postpone the fsync, to try to avoid massive I/O degradation. */
     if (!fc_unsafe_io)
-      namenode->flags |= fnnf_deferred_fsync;
+      namenode->flags |= FNNF_DEFERRED_FSYNC;
 
     pop_cleanup(ehflag_normaltidy); /* fd = open(path) */
     if (close(fd))
@@ -424,10 +424,10 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
   case TAR_FILETYPE_HARDLINK:
     varbuf_reset(&hardlinkfn);
     varbuf_add_str(&hardlinkfn, instdir);
-    linknode = findnamenode(te->linkname, 0);
+    linknode = fsys_hash_find_node(te->linkname, 0);
     varbuf_add_str(&hardlinkfn,
                    namenodetouse(linknode, tc->pkg, &tc->pkg->available)->name);
-    if (linknode->flags & (fnnf_deferred_rename | fnnf_new_conff))
+    if (linknode->flags & (FNNF_DEFERRED_RENAME | FNNF_NEW_CONFF))
       varbuf_add_str(&hardlinkfn, DPKGNEWEXT);
     varbuf_end_str(&hardlinkfn);
     if (link(hardlinkfn.buf, path))
@@ -454,7 +454,7 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
 
 static void
 tarobject_hash(struct tarcontext *tc, struct tar_entry *te,
-               struct filenamenode *namenode)
+               struct fsys_namenode *namenode)
 {
   if (te->type == TAR_FILETYPE_FILE) {
     struct dpkg_error err;
@@ -470,9 +470,9 @@ tarobject_hash(struct tarcontext *tc, struct tar_entry *te,
     namenode->newhash = newhash;
     debug(dbg_eachfiledetail, "tarobject file hash=%s", namenode->newhash);
   } else if (te->type == TAR_FILETYPE_HARDLINK) {
-    struct filenamenode *linknode;
+    struct fsys_namenode *linknode;
 
-    linknode = findnamenode(te->linkname, 0);
+    linknode = fsys_hash_find_node(te->linkname, 0);
     namenode->newhash = linknode->newhash;
     debug(dbg_eachfiledetail, "tarobject hardlink hash=%s", namenode->newhash);
   }
@@ -502,16 +502,21 @@ tarobject_set_mtime(struct tar_entry *te, const char *path)
 static void
 tarobject_set_perms(struct tar_entry *te, const char *path, struct file_stat *st)
 {
+  int rc;
+
   if (te->type == TAR_FILETYPE_FILE)
     return; /* Already handled using the file descriptor. */
 
   if (te->type == TAR_FILETYPE_SYMLINK) {
-    if (lchown(path, st->uid, st->gid))
+    rc = lchown(path, st->uid, st->gid);
+    if (forcible_nonroot_error(rc))
       ohshite(_("error setting ownership of symlink '%.255s'"), path);
   } else {
-    if (chown(path, st->uid, st->gid))
+    rc = chown(path, st->uid, st->gid);
+    if (forcible_nonroot_error(rc))
       ohshite(_("error setting ownership of '%.255s'"), path);
-    if (chmod(path, st->mode & ~S_IFMT))
+    rc = chmod(path, st->mode & ~S_IFMT);
+    if (forcible_nonroot_error(rc))
       ohshite(_("error setting permissions of '%.255s'"), path);
   }
 }
@@ -526,7 +531,7 @@ static void
 tarobject_matches(struct tarcontext *tc,
                   const char *fn_old, struct stat *stab, char *oldhash,
                   const char *fn_new, struct tar_entry *te,
-                  struct filenamenode *namenode)
+                  struct fsys_namenode *namenode)
 {
   char *linkname;
   ssize_t linksize;
@@ -539,16 +544,19 @@ tarobject_matches(struct tarcontext *tc,
     return;
   case TAR_FILETYPE_SYMLINK:
     /* Symlinks to existing dirs have already been dealt with, only
-     * reamin real symlinks where we can compare the target. */
+     * remain real symlinks where we can compare the target. */
     if (!S_ISLNK(stab->st_mode))
       break;
     linkname = m_malloc(stab->st_size + 1);
     linksize = readlink(fn_old, linkname, stab->st_size + 1);
     if (linksize < 0)
       ohshite(_("unable to read link '%.255s'"), fn_old);
-    else if (linksize != stab->st_size)
+    else if (linksize > stab->st_size)
       ohshit(_("symbolic link '%.250s' size has changed from %jd to %zd"),
              fn_old, (intmax_t)stab->st_size, linksize);
+    else if (linksize < stab->st_size)
+      warning(_("symbolic link '%.250s' size has changed from %jd to %zd"),
+             fn_old, (intmax_t)stab->st_size, linksize);
     linkname[linksize] = '\0';
     if (strcmp(linkname, te->linkname) == 0) {
       free(linkname);
@@ -573,7 +581,7 @@ tarobject_matches(struct tarcontext *tc,
     /* Fall through. */
   case TAR_FILETYPE_FILE:
     /* Only check metadata for non-conffiles. */
-    if (!(namenode->flags & fnnf_new_conff) &&
+    if (!(namenode->flags & FNNF_NEW_CONFF) &&
         !(S_ISREG(stab->st_mode) && te->size == stab->st_size))
       break;
     if (strcmp(oldhash, namenode->newhash) == 0)
@@ -632,7 +640,8 @@ linktosameexistingdir(const struct tar_entry *ti, const char *fname,
     varbuf_add_str(symlinkfn, instdir);
   } else {
     lastslash= strrchr(fname, '/');
-    assert(lastslash);
+    if (lastslash == NULL)
+      internerr("tar entry filename '%s' does not contain '/'", fname);
     varbuf_add_buf(symlinkfn, fname, (lastslash - fname) + 1);
   }
   varbuf_add_str(symlinkfn, ti->linkname);
@@ -654,14 +663,14 @@ linktosameexistingdir(const struct tar_entry *ti, const char *fname,
 }
 
 int
-tarobject(void *ctx, struct tar_entry *ti)
+tarobject(struct tar_archive *tar, struct tar_entry *ti)
 {
   static struct varbuf conffderefn, symlinkfn;
   const char *usename;
-  struct filenamenode *usenode;
+  struct fsys_namenode *usenode;
 
   struct conffile *conff;
-  struct tarcontext *tc = ctx;
+  struct tarcontext *tc = tar->ctx;
   bool existingdir, keepexisting;
   bool refcounting;
   char oldhash[MD5HASHLEN + 1];
@@ -669,7 +678,7 @@ tarobject(void *ctx, struct tar_entry *ti)
   ssize_t r;
   struct stat stab, stabtmp;
   struct file_stat nodestat;
-  struct fileinlist *nifd, **oldnifd;
+  struct fsys_namenode_list *nifd, **oldnifd;
   struct pkgset *divpkgset;
   struct pkginfo *otherpkg;
 
@@ -683,9 +692,9 @@ tarobject(void *ctx, struct tar_entry *ti)
    * The trailing ‘/’ put on the end of names in tarfiles has already
    * been stripped by tar_extractor(). */
   oldnifd = tc->newfiles_queue->tail;
-  nifd = tar_filenamenode_queue_push(tc->newfiles_queue,
-                                     findnamenode(ti->name, 0));
-  nifd->namenode->flags |= fnnf_new_inarchive;
+  nifd = tar_fsys_namenode_queue_push(tc->newfiles_queue,
+                                     fsys_hash_find_node(ti->name, 0));
+  nifd->namenode->flags |= FNNF_NEW_INARCHIVE;
 
   debug(dbg_eachfile,
         "tarobject ti->name='%s' mode=%lo owner=%u:%u type=%d(%c)"
@@ -728,12 +737,12 @@ tarobject(void *ctx, struct tar_entry *ti)
 
   trig_file_activate(usenode, tc->pkg);
 
-  if (nifd->namenode->flags & fnnf_new_conff) {
+  if (nifd->namenode->flags & FNNF_NEW_CONFF) {
     /* If it's a conffile we have to extract it next to the installed
      * version (i.e. we do the usual link-following). */
     if (conffderef(tc->pkg, &conffderefn, usename))
       usename= conffderefn.buf;
-    debug(dbg_conff, "tarobject fnnf_new_conff deref='%s'", usename);
+    debug(dbg_conff, "tarobject FNNF_NEW_CONFF deref='%s'", usename);
   }
 
   setupfnamevbs(usename);
@@ -800,10 +809,10 @@ tarobject(void *ctx, struct tar_entry *ti)
   keepexisting = false;
   refcounting = false;
   if (!existingdir) {
-    struct filepackages_iterator *iter;
+    struct fsys_node_pkgs_iter *iter;
 
-    iter = filepackages_iter_new(nifd->namenode);
-    while ((otherpkg = filepackages_iter_next(iter))) {
+    iter = fsys_node_pkgs_iter_new(nifd->namenode);
+    while ((otherpkg = fsys_node_pkgs_iter_next(iter))) {
       if (otherpkg == tc->pkg)
         continue;
       debug(dbg_eachfile, "tarobject ... found in %s",
@@ -844,16 +853,19 @@ tarobject(void *ctx, struct tar_entry *ti)
         continue;
       }
 
+      ensure_package_clientdata(otherpkg);
+
       /* Nope? Hmm, file conflict, perhaps. Check Replaces. */
       switch (otherpkg->clientdata->replacingfilesandsaid) {
       case 2:
         keepexisting = true;
+        /* Fall through. */
       case 1:
         continue;
       }
 
       /* Is the package with the conflicting file in the “config files only”
-       * state? If so it must be a config file and we can silenty take it
+       * state? If so it must be a config file and we can silently take it
        * over. */
       if (otherpkg->status == PKG_STAT_CONFIGFILES)
         continue;
@@ -864,21 +876,14 @@ tarobject(void *ctx, struct tar_entry *ti)
 
       /* Is the file an obsolete conffile in the other package
        * and a conffile in the new package? */
-      if ((nifd->namenode->flags & fnnf_new_conff) &&
+      if ((nifd->namenode->flags & FNNF_NEW_CONFF) &&
           !statr && S_ISREG(stab.st_mode)) {
         for (conff = otherpkg->installed.conffiles;
              conff;
              conff = conff->next) {
           if (!conff->obsolete)
             continue;
-          if (stat(conff->name, &stabtmp)) {
-            if (errno == ENOENT || errno == ENOTDIR || errno == ELOOP)
-              continue;
-            else
-              ohshite(_("cannot stat file '%s'"), conff->name);
-          }
-          if (stabtmp.st_dev == stab.st_dev &&
-              stabtmp.st_ino == stab.st_ino)
+          if (strcmp(conff->name, nifd->namenode->name) == 0)
             break;
         }
         if (conff) {
@@ -900,7 +905,7 @@ tarobject(void *ctx, struct tar_entry *ti)
                pkg_name(otherpkg, pnaw_nonambig),
                versiondescribe(&otherpkg->installed.version, vdew_nonambig));
         otherpkg->clientdata->replacingfilesandsaid = 2;
-        nifd->namenode->flags &= ~fnnf_new_inarchive;
+        nifd->namenode->flags &= ~FNNF_NEW_INARCHIVE;
         keepexisting = true;
       } else {
         /* At this point we are replacing something without a Replaces. */
@@ -921,20 +926,20 @@ tarobject(void *ctx, struct tar_entry *ti)
         }
       }
     }
-    filepackages_iter_free(iter);
+    fsys_node_pkgs_iter_free(iter);
   }
 
   if (keepexisting) {
-    if (nifd->namenode->flags & fnnf_new_conff)
-      nifd->namenode->flags |= fnnf_obs_conff;
-    tar_filenamenode_queue_pop(tc->newfiles_queue, oldnifd, nifd);
+    if (nifd->namenode->flags & FNNF_NEW_CONFF)
+      nifd->namenode->flags |= FNNF_OBS_CONFF;
+    tar_fsys_namenode_queue_pop(tc->newfiles_queue, oldnifd, nifd);
     tarobject_skip_entry(tc, ti);
     return 0;
   }
 
   if (filter_should_skip(ti)) {
-    nifd->namenode->flags &= ~fnnf_new_inarchive;
-    nifd->namenode->flags |= fnnf_filtered;
+    nifd->namenode->flags &= ~FNNF_NEW_INARCHIVE;
+    nifd->namenode->flags |= FNNF_FILTERED;
     tarobject_skip_entry(tc, ti);
 
     return 0;
@@ -948,7 +953,7 @@ tarobject(void *ctx, struct tar_entry *ti)
   if (refcounting) {
     debug(dbg_eachfiledetail, "tarobject hashing on-disk file '%s', refcounting",
           fnamevb.buf);
-    if (nifd->namenode->flags & fnnf_new_conff) {
+    if (nifd->namenode->flags & FNNF_NEW_CONFF) {
       md5hash_prev_conffile(tc->pkg, oldhash, fnamenewvb.buf, nifd->namenode);
     } else if (S_ISREG(stab.st_mode)) {
       md5hash(tc->pkg, oldhash, fnamevb.buf);
@@ -970,7 +975,7 @@ tarobject(void *ctx, struct tar_entry *ti)
     /* Now we start to do things that we need to be able to undo
      * if something goes wrong. Watch out for the CLEANUP comments to
      * keep an eye on what's installed on the disk at each point. */
-    push_cleanup(cu_installnew, ~ehflag_normaltidy, NULL, 0, 1, nifd->namenode);
+    push_cleanup(cu_installnew, ~ehflag_normaltidy, 1, nifd->namenode);
 
     /*
      * CLEANUP: Now we either have the old file on the disk, or not, in
@@ -1005,9 +1010,9 @@ tarobject(void *ctx, struct tar_entry *ti)
 
   /* First, check to see if it's a conffile. If so we don't install
    * it now - we leave it in .dpkg-new for --configure to take care of. */
-  if (nifd->namenode->flags & fnnf_new_conff) {
+  if (nifd->namenode->flags & FNNF_NEW_CONFF) {
     debug(dbg_conffdetail,"tarobject conffile extracted");
-    nifd->namenode->flags |= fnnf_elide_other_lists;
+    nifd->namenode->flags |= FNNF_ELIDE_OTHER_LISTS;
     return 0;
   }
 
@@ -1020,11 +1025,13 @@ tarobject(void *ctx, struct tar_entry *ti)
     if (ti->type == TAR_FILETYPE_DIR || S_ISDIR(stab.st_mode)) {
       /* One of the two is a directory - can't do atomic install. */
       debug(dbg_eachfiledetail,"tarobject directory, nonatomic");
-      nifd->namenode->flags |= fnnf_no_atomic_overwrite;
+      nifd->namenode->flags |= FNNF_NO_ATOMIC_OVERWRITE;
       if (rename(fnamevb.buf,fnametmpvb.buf))
         ohshite(_("unable to move aside '%.255s' to install new version"),
                 ti->name);
     } else if (S_ISLNK(stab.st_mode)) {
+      int rc;
+
       /* We can't make a symlink with two hardlinks, so we'll have to
        * copy it. (Pretend that making a copy of a symlink is the same
        * as linking to it.) */
@@ -1033,14 +1040,18 @@ tarobject(void *ctx, struct tar_entry *ti)
       r = readlink(fnamevb.buf, symlinkfn.buf, symlinkfn.size);
       if (r < 0)
         ohshite(_("unable to read link '%.255s'"), ti->name);
-      else if (r != stab.st_size)
+      else if (r > stab.st_size)
         ohshit(_("symbolic link '%.250s' size has changed from %jd to %zd"),
                fnamevb.buf, (intmax_t)stab.st_size, r);
+      else if (r < stab.st_size)
+        warning(_("symbolic link '%.250s' size has changed from %jd to %zd"),
+               fnamevb.buf, (intmax_t)stab.st_size, r);
       varbuf_trunc(&symlinkfn, r);
       varbuf_end_str(&symlinkfn);
       if (symlink(symlinkfn.buf,fnametmpvb.buf))
         ohshite(_("unable to make backup symlink for '%.255s'"), ti->name);
-      if (lchown(fnametmpvb.buf,stab.st_uid,stab.st_gid))
+      rc = lchown(fnametmpvb.buf, stab.st_uid, stab.st_gid);
+      if (forcible_nonroot_error(rc))
         ohshite(_("unable to chown backup symlink for '%.255s'"), ti->name);
       tarobject_set_se_context(fnamevb.buf, fnametmpvb.buf, stab.st_mode);
     } else {
@@ -1058,7 +1069,7 @@ tarobject(void *ctx, struct tar_entry *ti)
 
   if (ti->type == TAR_FILETYPE_FILE || ti->type == TAR_FILETYPE_HARDLINK ||
       ti->type == TAR_FILETYPE_SYMLINK) {
-    nifd->namenode->flags |= fnnf_deferred_rename;
+    nifd->namenode->flags |= FNNF_DEFERRED_RENAME;
 
     debug(dbg_eachfiledetail, "tarobject done and installation deferred");
   } else {
@@ -1072,8 +1083,8 @@ tarobject(void *ctx, struct tar_entry *ti)
      * remove the new file.
      */
 
-    nifd->namenode->flags |= fnnf_placed_on_disk;
-    nifd->namenode->flags |= fnnf_elide_other_lists;
+    nifd->namenode->flags |= FNNF_PLACED_ON_DISK;
+    nifd->namenode->flags |= FNNF_ELIDE_OTHER_LISTS;
 
     debug(dbg_eachfiledetail, "tarobject done and installed");
   }
@@ -1083,15 +1094,15 @@ tarobject(void *ctx, struct tar_entry *ti)
 
 #if defined(SYNC_FILE_RANGE_WAIT_BEFORE)
 static void
-tar_writeback_barrier(struct fileinlist *files, struct pkginfo *pkg)
+tar_writeback_barrier(struct fsys_namenode_list *files, struct pkginfo *pkg)
 {
-  struct fileinlist *cfile;
+  struct fsys_namenode_list *cfile;
 
   for (cfile = files; cfile; cfile = cfile->next) {
-    struct filenamenode *usenode;
+    struct fsys_namenode *usenode;
     int fd;
 
-    if (!(cfile->namenode->flags & fnnf_deferred_fsync))
+    if (!(cfile->namenode->flags & FNNF_DEFERRED_FSYNC))
       continue;
 
     usenode = namenodetouse(cfile->namenode, pkg, &pkg->available);
@@ -1111,30 +1122,30 @@ tar_writeback_barrier(struct fileinlist *files, struct pkginfo *pkg)
 }
 #else
 static void
-tar_writeback_barrier(struct fileinlist *files, struct pkginfo *pkg)
+tar_writeback_barrier(struct fsys_namenode_list *files, struct pkginfo *pkg)
 {
 }
 #endif
 
 void
-tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
+tar_deferred_extract(struct fsys_namenode_list *files, struct pkginfo *pkg)
 {
-  struct fileinlist *cfile;
-  struct filenamenode *usenode;
+  struct fsys_namenode_list *cfile;
+  struct fsys_namenode *usenode;
 
   tar_writeback_barrier(files, pkg);
 
   for (cfile = files; cfile; cfile = cfile->next) {
     debug(dbg_eachfile, "deferred extract of '%.255s'", cfile->namenode->name);
 
-    if (!(cfile->namenode->flags & fnnf_deferred_rename))
+    if (!(cfile->namenode->flags & FNNF_DEFERRED_RENAME))
       continue;
 
     usenode = namenodetouse(cfile->namenode, pkg, &pkg->available);
 
     setupfnamevbs(usenode->name);
 
-    if (cfile->namenode->flags & fnnf_deferred_fsync) {
+    if (cfile->namenode->flags & FNNF_DEFERRED_FSYNC) {
       int fd;
 
       debug(dbg_eachfiledetail, "deferred extract needs fsync");
@@ -1147,7 +1158,7 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
       if (close(fd))
         ohshite(_("error closing/writing '%.255s'"), fnamenewvb.buf);
 
-      cfile->namenode->flags &= ~fnnf_deferred_fsync;
+      cfile->namenode->flags &= ~FNNF_DEFERRED_FSYNC;
     }
 
     debug(dbg_eachfiledetail, "deferred extract needs rename");
@@ -1156,7 +1167,7 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
       ohshite(_("unable to install new version of '%.255s'"),
               cfile->namenode->name);
 
-    cfile->namenode->flags &= ~fnnf_deferred_rename;
+    cfile->namenode->flags &= ~FNNF_DEFERRED_RENAME;
 
     /*
      * CLEANUP: Now the new file is in the destination file, and the
@@ -1165,8 +1176,8 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
      * remove the new file.
      */
 
-    cfile->namenode->flags |= fnnf_placed_on_disk;
-    cfile->namenode->flags |= fnnf_elide_other_lists;
+    cfile->namenode->flags |= FNNF_PLACED_ON_DISK;
+    cfile->namenode->flags |= FNNF_ELIDE_OTHER_LISTS;
 
     debug(dbg_eachfiledetail, "deferred extract done and installed");
   }
@@ -1179,7 +1190,7 @@ enqueue_deconfigure(struct pkginfo *pkg, struct pkginfo *pkg_removal)
 
   ensure_package_clientdata(pkg);
   pkg->clientdata->istobe = PKG_ISTOBE_DECONFIGURE;
-  newdeconf = m_malloc(sizeof(struct pkg_deconf_list));
+  newdeconf = m_malloc(sizeof(*newdeconf));
   newdeconf->next = deconfigure;
   newdeconf->pkg = pkg;
   newdeconf->pkg_removal = pkg_removal;
@@ -1271,7 +1282,11 @@ void check_breaks(struct dependency *dep, struct pkginfo *pkg,
     char action[512];
 
     ensure_package_clientdata(fixbydeconf);
-    assert(fixbydeconf->clientdata->istobe == PKG_ISTOBE_NORMAL);
+
+    if (fixbydeconf->clientdata->istobe != PKG_ISTOBE_NORMAL)
+      internerr("package %s being fixed by deconf is not to be normal, "
+                "is to be %d",
+                pkg_name(pkg, pnaw_always), fixbydeconf->clientdata->istobe);
 
     sprintf(action, _("installation of %.250s"),
             pkgbin_name(pkg, &pkg->available, pnaw_nonambig));
@@ -1333,8 +1348,13 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
             fixbyrm->want != PKG_WANT_HOLD) ||
            does_replace(pkg, &pkg->available, fixbyrm, &fixbyrm->installed)) &&
           (!fixbyrm->installed.essential || fc_removeessential)))) {
-      assert(fixbyrm->clientdata->istobe == PKG_ISTOBE_NORMAL ||
-             fixbyrm->clientdata->istobe == PKG_ISTOBE_DECONFIGURE);
+
+      if (fixbyrm->clientdata->istobe != PKG_ISTOBE_NORMAL &&
+          fixbyrm->clientdata->istobe != PKG_ISTOBE_DECONFIGURE)
+        internerr("package %s to be fixed by removal is not to be normal "
+                  "nor deconfigure, is to be %d",
+                  pkg_name(pkg, pnaw_always), fixbyrm->clientdata->istobe);
+
       fixbyrm->clientdata->istobe = PKG_ISTOBE_REMOVE;
       notice(_("considering removing %s in favour of %s ..."),
              pkg_name(fixbyrm, pnaw_nonambig),
