@@ -2,8 +2,8 @@
  * dpkg - main program for package management
  * main.h - external definitions for this program
  *
- * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2006, 2008-2015 Guillem Jover <guillem@debian.org>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
+ * Copyright © 2006, 2008-2016 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,10 +25,9 @@
 #include <dpkg/debug.h>
 #include <dpkg/pkg-list.h>
 
-/* These two are defined in filesdb.h. */
-struct fileinlist;
-struct filenamenode_queue;
-struct filenamenode;
+/* These two are defined in <dpkg/fsys.h>. */
+struct fsys_namenode_list;
+struct fsys_namenode;
 
 enum pkg_istobe {
 	/** Package is to be left in a normal state. */
@@ -57,22 +56,9 @@ struct perpackagestate {
 
   bool enqueued;
 
-  /**
-   * filelistvalid  files  Meaning
-   * -------------  -----  -------
-   * false          NULL   Not read yet, must do so if want them.
-   * false          !NULL  Read, but rewritten and now out of date. If want
-   *                         info must throw away old and reread file.
-   * true           !NULL  Read, all is OK.
-   * true           NULL   Read OK, but, there were no files.
-   */
-  bool fileslistvalid;
-  struct fileinlist *files;
   int replacingfilesandsaid;
   int cmdline_seen;
 
-  off_t listfile_phys_offs;
-
   /** Non-NULL iff in trigproc.c:deferred. */
   struct pkg_list *trigprocdeferred;
 };
@@ -102,7 +88,6 @@ enum action {
 	act_arch_add,
 	act_arch_remove,
 	act_printarch,
-	act_printinstarch,
 	act_printforeignarches,
 
 	act_assertpredep,
@@ -112,6 +97,11 @@ enum action {
 	act_assertmultiarch,
 	act_assertverprovides,
 
+	act_validate_pkgname,
+	act_validate_trigname,
+	act_validate_archname,
+	act_validate_version,
+
 	act_audit,
 	act_unpackchk,
 	act_predeppackage,
@@ -142,6 +132,7 @@ extern int fc_conff_ask;
 extern int fc_badverify;
 extern int fc_badversion;
 extern int fc_unsafe_io;
+extern int fc_script_chrootless;
 
 extern bool abort_processing;
 extern int errabort;
@@ -150,19 +141,23 @@ extern struct pkg_list *ignoredependss;
 
 struct invoke_hook {
 	struct invoke_hook *next;
-	const char *command;
+	char *command;
 };
 
+struct invoke_list {
+	struct invoke_hook *head, **tail;
+};
+
+/* from perpkgstate.c */
+
+void ensure_package_clientdata(struct pkginfo *pkg);
+
 /* from archives.c */
 
 int archivefiles(const char *const *argv);
 void process_archive(const char *filename);
 bool wanttoinstall(struct pkginfo *pkg);
 
-struct fileinlist *
-filenamenode_queue_push(struct filenamenode_queue *queue,
-                        struct filenamenode *namenode);
-
 /* from update.c */
 
 int forgetold(const char *const *argv);
@@ -178,6 +173,10 @@ int assertlongfilenames(const char *const *argv);
 int assertmulticonrep(const char *const *argv);
 int assertmultiarch(const char *const *argv);
 int assertverprovides(const char *const *argv);
+int validate_pkgname(const char *const *argv);
+int validate_trigname(const char *const *argv);
+int validate_archname(const char *const *argv);
+int validate_version(const char *const *argv);
 int predeppackage(const char *const *argv);
 int printarch(const char *const *argv);
 int printinstarch(const char *const *argv);
@@ -218,7 +217,48 @@ enum dep_check breakses_ok(struct pkginfo *pkg, struct varbuf *aemsgs);
 void deferred_remove(struct pkginfo *pkg);
 void deferred_configure(struct pkginfo *pkg);
 
-extern int sincenothing, dependtry;
+/*
+ * During the packages queue processing, the algorithm for deciding what to
+ * configure first is as follows:
+ *
+ * Loop through all packages doing a ‘try 1’ until we've been round and
+ * nothing has been done, then do ‘try 2’, and subsequent ones likewise.
+ * The incrementing of ‘dependtry’ is done by process_queue().
+ *
+ * Try 1:
+ *   Are all dependencies of this package done? If so, do it.
+ *   Are any of the dependencies missing or the wrong version?
+ *     If so, abort (unless --force-depends, in which case defer).
+ *   Will we need to configure a package we weren't given as an
+ *     argument? If so, abort ─ except if --force-configure-any,
+ *     in which case we add the package to the argument list.
+ *   If none of the above, defer the package.
+ *
+ * Try 2:
+ *   Find a cycle and break it (see above).
+ *   Do as for try 1.
+ *
+ * Try 3:
+ *   Start processing triggers if necessary.
+ *   Do as for try 2.
+ *
+ * Try 4 (only if --force-depends-version):
+ *   Same as for try 2, but don't mind version number in dependencies.
+ *
+ * Try 5 (only if --force-depends):
+ *   Do anyway.
+ */
+enum dependtry {
+	DEPEND_TRY_NORMAL = 1,
+	DEPEND_TRY_CYCLES = 2,
+	DEPEND_TRY_TRIGGERS = 3,
+	DEPEND_TRY_FORCE_DEPENDS_VERSION = 4,
+	DEPEND_TRY_FORCE_DEPENDS = 5,
+	DEPEND_TRY_LAST,
+};
+
+extern enum dependtry dependtry;
+extern int sincenothing;
 
 /* from cleanup.c (most of these are declared in archives.h) */
 
@@ -229,6 +269,7 @@ void cu_prermremove(int argc, void **argv);
 void print_error_perpackage(const char *emsg, const void *data);
 void print_error_perarchive(const char *emsg, const void *data);
 void forcibleerr(int forceflag, const char *format, ...) DPKG_ATTR_PRINTF(2);
+int forcible_nonroot_error(int rc);
 int reportbroken_retexitstatus(int ret);
 bool skip_due_to_hold(struct pkginfo *pkg);
 
@@ -240,12 +281,15 @@ bool ignore_depends(struct pkginfo *pkg);
 bool force_breaks(struct deppossi *possi);
 bool force_depends(struct deppossi *possi);
 bool force_conflicts(struct deppossi *possi);
-void conffile_mark_obsolete(struct pkginfo *pkg, struct filenamenode *namenode);
+void
+conffile_mark_obsolete(struct pkginfo *pkg, struct fsys_namenode *namenode);
 void pkg_conffiles_mark_old(struct pkginfo *pkg);
+bool find_command(const char *prog);
 void checkpath(void);
 
-struct filenamenode *namenodetouse(struct filenamenode *namenode,
-                                   struct pkginfo *pkg, struct pkgbin *pkgbin);
+struct fsys_namenode *
+namenodetouse(struct fsys_namenode *namenode,
+              struct pkginfo *pkg, struct pkgbin *pkgbin);
 
 int maintscript_installed(struct pkginfo *pkg, const char *scriptname,
                           const char *desc, ...) DPKG_ATTR_SENTINEL;
@@ -265,10 +309,13 @@ int maintscript_postinst(struct pkginfo *pkg, ...) DPKG_ATTR_SENTINEL;
 void post_postinst_tasks(struct pkginfo *pkg, enum pkgstatus new_status);
 
 void clear_istobes(void);
-bool dir_is_used_by_others(struct filenamenode *namenode, struct pkginfo *pkg);
-bool dir_is_used_by_pkg(struct filenamenode *namenode, struct pkginfo *pkg,
-                        struct fileinlist *list);
-bool dir_has_conffiles(struct filenamenode *namenode, struct pkginfo *pkg);
+bool
+dir_is_used_by_others(struct fsys_namenode *namenode, struct pkginfo *pkg);
+bool
+dir_is_used_by_pkg(struct fsys_namenode *namenode, struct pkginfo *pkg,
+                   struct fsys_namenode_list *list);
+bool
+dir_has_conffiles(struct fsys_namenode *namenode, struct pkginfo *pkg);
 
 void log_action(const char *action, struct pkginfo *pkg, struct pkgbin *pkgbin);
 
@@ -281,8 +328,10 @@ void dpkg_selabel_close(void);
 /* from trigproc.c */
 
 enum trigproc_type {
-	/** Opportunistic trigger processing. */
-	TRIGPROC_TRY,
+	/** Opportunistic deferred trigger processing. */
+	TRIGPROC_TRY_DEFERRED,
+	/** Opportunistic queued trigger processing. */
+	TRIGPROC_TRY_QUEUED,
 	/** Required trigger processing. */
 	TRIGPROC_REQUIRED,
 };
