@@ -2,8 +2,8 @@
  * dpkg - main program for package management
  * remove.c - functionality for removing packages
  *
- * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2007-2014 Guillem Jover <guillem@debian.org>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
+ * Copyright © 2007-2015 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <sys/stat.h>
 
 #include <errno.h>
-#include <ctype.h>
 #include <string.h>
 #include <fcntl.h>
 #include <dirent.h>
@@ -35,15 +34,17 @@
 #include <stdio.h>
 
 #include <dpkg/i18n.h>
+#include <dpkg/c-ctype.h>
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/pkg.h>
+#include <dpkg/path.h>
 #include <dpkg/dir.h>
 #include <dpkg/options.h>
 #include <dpkg/triglib.h>
+#include <dpkg/db-ctrl.h>
+#include <dpkg/db-fsys.h>
 
-#include "infodb.h"
-#include "filesdb.h"
 #include "main.h"
 
 /*
@@ -56,7 +57,7 @@ static void checkforremoval(struct pkginfo *pkgtoremove,
   struct deppossi *possi;
   struct pkginfo *depender;
   enum dep_check ok;
-  int before;
+  struct varbuf_state raemsgs_state;
 
   for (possi = pkgdepcheck->depended.installed; possi; possi = possi->rev_next) {
     if (possi->up->type != dep_depends && possi->up->type != dep_predepends) continue;
@@ -70,15 +71,19 @@ static void checkforremoval(struct pkginfo *pkgtoremove,
             pkg_name(depender, pnaw_always));
       continue;
     }
-    if (dependtry > 1) { if (findbreakcycle(pkgtoremove)) sincenothing= 0; }
-    before= raemsgs->used;
+    if (dependtry >= DEPEND_TRY_CYCLES) {
+      if (findbreakcycle(pkgtoremove))
+        sincenothing = 0;
+    }
+    varbuf_snapshot(raemsgs, &raemsgs_state);
     ok= dependencies_ok(depender,pkgtoremove,raemsgs);
     if (ok == DEP_CHECK_HALT &&
+        depender->clientdata &&
         depender->clientdata->istobe == PKG_ISTOBE_REMOVE)
       ok = DEP_CHECK_DEFER;
     if (ok == DEP_CHECK_DEFER)
       /* Don't burble about reasons for deferral. */
-      varbuf_trunc(raemsgs, before);
+      varbuf_rollback(raemsgs, &raemsgs_state);
     if (ok < *rokp) *rokp= ok;
   }
 }
@@ -101,6 +106,8 @@ void deferred_remove(struct pkginfo *pkg) {
       modstatdb_note(pkg);
   }
 
+  ensure_package_clientdata(pkg);
+
   if (pkg->status == PKG_STAT_NOTINSTALLED) {
     sincenothing = 0;
     warning(_("ignoring request to remove %.250s which isn't installed"),
@@ -157,7 +164,7 @@ void deferred_remove(struct pkginfo *pkg) {
                   " reinstall it before attempting a removal"));
 
   ensure_allinstfiles_available();
-  filesdbinit();
+  fsys_hash_init();
 
   if (f_noact) {
     printf(_("Would remove or purge %s (%s) ...\n"),
@@ -168,11 +175,16 @@ void deferred_remove(struct pkginfo *pkg) {
     return;
   }
 
-  oldconffsetflags(pkg->installed.conffiles);
+  pkg_conffiles_mark_old(pkg);
+
+  /* Only print and log removal action once. This avoids duplication when
+   * using --remove and --purge in sequence. */
+  if (pkg->status > PKG_STAT_CONFIGFILES) {
+    printf(_("Removing %s (%s) ...\n"), pkg_name(pkg, pnaw_nonambig),
+           versiondescribe(&pkg->installed.version, vdew_nonambig));
+    log_action("remove", pkg, &pkg->installed);
+  }
 
-  printf(_("Removing %s (%s) ...\n"), pkg_name(pkg, pnaw_nonambig),
-         versiondescribe(&pkg->installed.version, vdew_nonambig));
-  log_action("remove", pkg, &pkg->installed);
   trig_activate_packageprocessing(pkg);
   if (pkg->status >= PKG_STAT_HALFCONFIGURED) {
     static enum pkgstatus oldpkgstatus;
@@ -180,7 +192,7 @@ void deferred_remove(struct pkginfo *pkg) {
     oldpkgstatus= pkg->status;
     pkg_set_status(pkg, PKG_STAT_HALFCONFIGURED);
     modstatdb_note(pkg);
-    push_cleanup(cu_prermremove, ~ehflag_normaltidy, NULL, 0, 2,
+    push_cleanup(cu_prermremove, ~ehflag_normaltidy, 2,
                  (void *)pkg, (void *)&oldpkgstatus);
     maintscript_installed(pkg, PRERMFILE, "pre-removal", "remove", NULL);
 
@@ -191,10 +203,13 @@ void deferred_remove(struct pkginfo *pkg) {
   removal_bulk(pkg);
 }
 
-static void push_leftover(struct fileinlist **leftoverp,
-                          struct filenamenode *namenode) {
-  struct fileinlist *newentry;
-  newentry= nfmalloc(sizeof(struct fileinlist));
+static void
+push_leftover(struct fsys_namenode_list **leftoverp,
+              struct fsys_namenode *namenode)
+{
+  struct fsys_namenode_list *newentry;
+
+  newentry = nfmalloc(sizeof(*newentry));
   newentry->next= *leftoverp;
   newentry->namenode= namenode;
   *leftoverp= newentry;
@@ -211,23 +226,23 @@ removal_bulk_remove_file(const char *filename, const char *filetype)
   debug(dbg_stupidlyverbose, "removal_bulk info not postrm or list");
 
   if (unlink(filename))
-    ohshite(_("unable to delete control info file `%.250s'"), filename);
+    ohshite(_("unable to delete control info file '%.250s'"), filename);
 
   debug(dbg_scripts, "removal_bulk info unlinked %s", filename);
 }
 
 static bool
-removal_bulk_file_is_shared(struct pkginfo *pkg, struct filenamenode *namenode)
+removal_bulk_file_is_shared(struct pkginfo *pkg, struct fsys_namenode *namenode)
 {
-  struct filepackages_iterator *iter;
+  struct fsys_node_pkgs_iter *iter;
   struct pkginfo *otherpkg;
   bool shared = false;
 
   if (pkgset_installed_instances(pkg->set) <= 1)
     return false;
 
-  iter = filepackages_iter_new(namenode);
-  while ((otherpkg = filepackages_iter_next(iter))) {
+  iter = fsys_node_pkgs_iter_new(namenode);
+  while ((otherpkg = fsys_node_pkgs_iter_next(iter))) {
     if (otherpkg == pkg)
       continue;
     if (otherpkg->set != pkg->set)
@@ -238,7 +253,7 @@ removal_bulk_file_is_shared(struct pkginfo *pkg, struct filenamenode *namenode)
     shared = true;
     break;
   }
-  filepackages_iter_free(iter);
+  fsys_node_pkgs_iter_free(iter);
 
   return shared;
 }
@@ -246,21 +261,21 @@ removal_bulk_file_is_shared(struct pkginfo *pkg, struct filenamenode *namenode)
 static void
 removal_bulk_remove_files(struct pkginfo *pkg)
 {
-  int before;
-  struct reversefilelistiter rlistit;
-  struct fileinlist *leftover;
-  struct filenamenode *namenode;
+  struct fsys_hash_rev_iter rev_iter;
+  struct fsys_namenode_list *leftover;
+  struct fsys_namenode *namenode;
   static struct varbuf fnvb;
+  struct varbuf_state fnvb_state;
   struct stat stab;
 
     pkg_set_status(pkg, PKG_STAT_HALFINSTALLED);
     modstatdb_note(pkg);
     push_checkpoint(~ehflag_bombout, ehflag_normaltidy);
 
-    reversefilelist_init(&rlistit,pkg->clientdata->files);
+    fsys_hash_rev_iter_init(&rev_iter, pkg->files);
     leftover = NULL;
-    while ((namenode= reversefilelist_next(&rlistit))) {
-      struct filenamenode *usenode;
+    while ((namenode = fsys_hash_rev_iter_next(&rev_iter))) {
+      struct fsys_namenode *usenode;
       bool is_dir;
 
       debug(dbg_eachfile, "removal_bulk '%s' flags=%o",
@@ -272,7 +287,7 @@ removal_bulk_remove_files(struct pkginfo *pkg)
       varbuf_add_str(&fnvb, instdir);
       varbuf_add_str(&fnvb, usenode->name);
       varbuf_end_str(&fnvb);
-      before= fnvb.used;
+      varbuf_snapshot(&fnvb, &fnvb_state);
 
       is_dir = stat(fnvb.buf, &stab) == 0 && S_ISDIR(stab.st_mode);
 
@@ -283,7 +298,7 @@ removal_bulk_remove_files(struct pkginfo *pkg)
         continue;
 
       /* Non-shared conffiles are kept. */
-      if (namenode->flags & fnnf_old_conff) {
+      if (namenode->flags & FNNF_OLD_CONFF) {
         push_leftover(&leftover, namenode);
         continue;
       }
@@ -314,19 +329,19 @@ removal_bulk_remove_files(struct pkginfo *pkg)
 
       trig_path_activate(usenode, pkg);
 
-      varbuf_trunc(&fnvb, before);
+      varbuf_rollback(&fnvb, &fnvb_state);
       varbuf_add_str(&fnvb, DPKGTEMPEXT);
       varbuf_end_str(&fnvb);
       debug(dbg_eachfiledetail, "removal_bulk cleaning temp '%s'", fnvb.buf);
-      ensure_pathname_nonexisting(fnvb.buf);
+      path_remove_tree(fnvb.buf);
 
-      varbuf_trunc(&fnvb, before);
+      varbuf_rollback(&fnvb, &fnvb_state);
       varbuf_add_str(&fnvb, DPKGNEWEXT);
       varbuf_end_str(&fnvb);
       debug(dbg_eachfiledetail, "removal_bulk cleaning new '%s'", fnvb.buf);
-      ensure_pathname_nonexisting(fnvb.buf);
+      path_remove_tree(fnvb.buf);
 
-      varbuf_trunc(&fnvb, before);
+      varbuf_rollback(&fnvb, &fnvb_state);
       varbuf_end_str(&fnvb);
 
       debug(dbg_eachfiledetail, "removal_bulk removing '%s'", fnvb.buf);
@@ -344,7 +359,8 @@ removal_bulk_remove_files(struct pkginfo *pkg)
         push_leftover(&leftover,namenode);
         continue;
       }
-      if (errno != ENOTDIR) ohshite(_("cannot remove `%.250s'"),fnvb.buf);
+      if (errno != ENOTDIR)
+        ohshite(_("cannot remove '%.250s'"), fnvb.buf);
       debug(dbg_eachfiledetail, "removal_bulk unlinking '%s'", fnvb.buf);
       if (secure_unlink(fnvb.buf))
         ohshite(_("unable to securely remove '%.250s'"), fnvb.buf);
@@ -367,9 +383,9 @@ removal_bulk_remove_files(struct pkginfo *pkg)
 }
 
 static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
-  struct reversefilelistiter rlistit;
-  struct fileinlist *leftover;
-  struct filenamenode *namenode;
+  struct fsys_hash_rev_iter rev_iter;
+  struct fsys_namenode_list *leftover;
+  struct fsys_namenode *namenode;
   static struct varbuf fnvb;
   struct stat stab;
 
@@ -379,14 +395,14 @@ static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
   modstatdb_note(pkg);
   push_checkpoint(~ehflag_bombout, ehflag_normaltidy);
 
-  reversefilelist_init(&rlistit,pkg->clientdata->files);
+  fsys_hash_rev_iter_init(&rev_iter, pkg->files);
   leftover = NULL;
-  while ((namenode= reversefilelist_next(&rlistit))) {
-    struct filenamenode *usenode;
+  while ((namenode = fsys_hash_rev_iter_next(&rev_iter))) {
+    struct fsys_namenode *usenode;
 
     debug(dbg_eachfile, "removal_bulk '%s' flags=%o",
           namenode->name, namenode->flags);
-    if (namenode->flags & fnnf_old_conff) {
+    if (namenode->flags & FNNF_OLD_CONFF) {
       /* This can only happen if removal_bulk_remove_configfiles() got
        * interrupted half way. */
       debug(dbg_eachfiledetail, "removal_bulk expecting only left over dirs, "
@@ -437,7 +453,8 @@ static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
       push_leftover(&leftover,namenode);
       continue;
     }
-    if (errno != ENOTDIR) ohshite(_("cannot remove `%.250s'"),fnvb.buf);
+    if (errno != ENOTDIR)
+      ohshite(_("cannot remove '%.250s'"), fnvb.buf);
 
     if (lstat(fnvb.buf, &stab) == 0 && S_ISLNK(stab.st_mode)) {
       debug(dbg_eachfiledetail, "removal_bulk is a symlink to a directory");
@@ -459,11 +476,11 @@ static void removal_bulk_remove_leftover_dirs(struct pkginfo *pkg) {
 
 static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
   static const char *const removeconffexts[] = { REMOVECONFFEXTS, NULL };
-  int rc, removevbbase;
+  int rc;
   int conffnameused, conffbasenamelen;
   char *conffbasename;
   struct conffile *conff, **lconffp;
-  struct fileinlist *searchfile;
+  struct fsys_namenode_list *searchfile;
   DIR *dsd;
   struct dirent *de;
   char *p;
@@ -487,7 +504,7 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
      * are involved in diversions, except if we are the package doing the
      * diverting. */
     for (lconffp = &pkg->installed.conffiles; (conff = *lconffp) != NULL; ) {
-      for (searchfile= pkg->clientdata->files;
+      for (searchfile = pkg->files;
            searchfile && strcmp(searchfile->namenode->name,conff->name);
            searchfile= searchfile->next);
       if (!searchfile) {
@@ -511,7 +528,10 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
     modstatdb_note(pkg);
 
     for (conff= pkg->installed.conffiles; conff; conff= conff->next) {
+      struct fsys_namenode *namenode, *usenode;
     static struct varbuf fnvb, removevb;
+      struct varbuf_state removevb_state;
+
       if (conff->obsolete) {
 	debug(dbg_conffdetail, "removal_bulk conffile obsolete %s",
 	      conff->name);
@@ -522,28 +542,35 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
             conff->name, rc == -1 ? "<rc == -1>" : fnvb.buf);
       if (rc == -1)
         continue;
+
+      namenode = fsys_hash_find_node(conff->name, 0);
+      usenode = namenodetouse(namenode, pkg, &pkg->installed);
+
+      trig_path_activate(usenode, pkg);
+
       conffnameused = fnvb.used;
       if (unlink(fnvb.buf) && errno != ENOENT && errno != ENOTDIR)
-        ohshite(_("cannot remove old config file `%.250s' (= `%.250s')"),
+        ohshite(_("cannot remove old config file '%.250s' (= '%.250s')"),
                 conff->name, fnvb.buf);
       p= strrchr(fnvb.buf,'/'); if (!p) continue;
       *p = '\0';
       varbuf_reset(&removevb);
       varbuf_add_str(&removevb, fnvb.buf);
       varbuf_add_char(&removevb, '/');
-      removevbbase= removevb.used;
       varbuf_end_str(&removevb);
+      varbuf_snapshot(&removevb, &removevb_state);
+
       dsd= opendir(removevb.buf);
       if (!dsd) {
         int e=errno;
         debug(dbg_conffdetail, "removal_bulk conffile no dsd %s %s",
               fnvb.buf, strerror(e)); errno= e;
         if (errno == ENOENT || errno == ENOTDIR) continue;
-        ohshite(_("cannot read config file dir `%.250s' (from `%.250s')"),
+        ohshite(_("cannot read config file directory '%.250s' (from '%.250s')"),
                 fnvb.buf, conff->name);
       }
       debug(dbg_conffdetail, "removal_bulk conffile cleaning dsd %s", fnvb.buf);
-      push_cleanup(cu_closedir, ~0, NULL, 0, 1, (void *)dsd);
+      push_cleanup(cu_closedir, ~0, 1, (void *)dsd);
       *p= '/';
       conffbasenamelen= strlen(++p);
       conffbasename= fnvb.buf+conffnameused-conffbasenamelen;
@@ -558,7 +585,8 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
               goto yes_remove;
           p= de->d_name+conffbasenamelen;
           if (*p++ == '~') {
-            while (*p && cisdigit(*p)) p++;
+            while (*p && c_isdigit(*p))
+              p++;
             if (*p == '~' && !*++p) goto yes_remove;
           }
         }
@@ -570,21 +598,21 @@ static void removal_bulk_remove_configfiles(struct pkginfo *pkg) {
         debug(dbg_stupidlyverbose, "removal_bulk conffile dsd entry not it");
         continue;
       yes_remove:
-        varbuf_trunc(&removevb, removevbbase);
+        varbuf_rollback(&removevb, &removevb_state);
         varbuf_add_str(&removevb, de->d_name);
         varbuf_end_str(&removevb);
         debug(dbg_conffdetail, "removal_bulk conffile dsd entry removing '%s'",
               removevb.buf);
         if (unlink(removevb.buf) && errno != ENOENT && errno != ENOTDIR)
-          ohshite(_("cannot remove old backup config file `%.250s' (of `%.250s')"),
+          ohshite(_("cannot remove old backup config file '%.250s' (of '%.250s')"),
                   removevb.buf, conff->name);
       }
       pop_cleanup(ehflag_normaltidy); /* closedir */
     }
 
     /* Remove the conffiles from the file list file. */
-    write_filelist_except(pkg, &pkg->installed, pkg->clientdata->files,
-                          fnnf_old_conff);
+    write_filelist_except(pkg, &pkg->installed, pkg->files,
+                          FNNF_OLD_CONFF);
 
     pkg->installed.conffiles = NULL;
     modstatdb_note(pkg);
