@@ -25,7 +25,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
@@ -43,9 +42,9 @@
 #include <dpkg/subproc.h>
 #include <dpkg/command.h>
 #include <dpkg/triglib.h>
+#include <dpkg/db-ctrl.h>
+#include <dpkg/db-fsys.h>
 
-#include "filesdb.h"
-#include "infodb.h"
 #include "main.h"
 
 void
@@ -107,6 +106,8 @@ maintscript_pre_exec(struct command *cmd)
 		changedir = "/";
 
 	if (instdirlen > 0 && !fc_script_chrootless) {
+		int rc;
+
 		if (strncmp(admindir, instdir, instdirlen) != 0)
 			ohshit(_("admindir must be inside instdir for dpkg to work properly"));
 		if (setenv("DPKG_ADMINDIR", admindir + instdirlen, 1) < 0)
@@ -114,7 +115,12 @@ maintscript_pre_exec(struct command *cmd)
 		if (setenv("DPKG_ROOT", "", 1) < 0)
 			ohshite(_("unable to setenv for subprocesses"));
 
-		if (chroot(instdir))
+		rc = chroot(instdir);
+		if (rc && fc_nonroot && errno == EPERM)
+			ohshit(_("not enough privileges to change root "
+			         "directory with --force-not-root, consider "
+			         "using --force-script-chrootless?"));
+		else if (rc)
 			ohshite(_("failed to chroot to '%.250s'"), instdir);
 	}
 	/* Switch to a known good directory to give the maintainer script
@@ -137,7 +143,10 @@ maintscript_pre_exec(struct command *cmd)
 	if (instdirlen == 0 || fc_script_chrootless)
 		return cmd->filename;
 
-	assert(strlen(cmd->filename) >= instdirlen);
+	if (strlen(cmd->filename) < instdirlen)
+		internerr("maintscript name '%s' length < instdir length %zd",
+		          cmd->filename, instdirlen);
+
 	return cmd->filename + instdirlen;
 }
 
@@ -169,7 +178,7 @@ maintscript_exec(struct pkginfo *pkg, struct pkgbin *pkgbin,
 
 	setexecute(cmd->filename, stab);
 
-	push_cleanup(cu_post_script_tasks, ehflag_bombout, NULL, 0, 0);
+	push_cleanup(cu_post_script_tasks, ehflag_bombout, 0);
 
 	pid = subproc_fork();
 	if (pid == 0) {
@@ -212,10 +221,11 @@ vmaintscript_installed(struct pkginfo *pkg, const char *scriptname,
 	struct command cmd;
 	const char *scriptpath;
 	struct stat stab;
-	char buf[100];
+	char *buf;
 
 	scriptpath = pkg_infodb_get_file(pkg, &pkg->installed, scriptname);
-	sprintf(buf, _("installed %s script"), desc);
+	m_asprintf(&buf, _("installed %s package %s script"),
+	           pkg_name(pkg, pnaw_nonambig), desc);
 
 	command_init(&cmd, scriptpath, buf);
 	command_add_arg(&cmd, scriptname);
@@ -223,10 +233,12 @@ vmaintscript_installed(struct pkginfo *pkg, const char *scriptname,
 
 	if (stat(scriptpath, &stab)) {
 		command_destroy(&cmd);
+
 		if (errno == ENOENT) {
 			debug(dbg_scripts,
 			      "vmaintscript_installed nonexistent %s",
 			      scriptname);
+			free(buf);
 			return 0;
 		}
 		ohshite(_("unable to stat %s '%.250s'"), buf, scriptpath);
@@ -234,6 +246,7 @@ vmaintscript_installed(struct pkginfo *pkg, const char *scriptname,
 	maintscript_exec(pkg, &pkg->installed, &cmd, &stab, 0);
 
 	command_destroy(&cmd);
+	free(buf);
 
 	return 1;
 }
@@ -282,10 +295,11 @@ maintscript_new(struct pkginfo *pkg, const char *scriptname,
 	struct command cmd;
 	struct stat stab;
 	va_list args;
-	char buf[100];
+	char *buf;
 
 	strcpy(cidirrest, scriptname);
-	sprintf(buf, _("new %s script"), desc);
+	m_asprintf(&buf, _("new %s package %s script"),
+	           pkg_name(pkg, pnaw_nonambig), desc);
 
 	va_start(args, cidirrest);
 	command_init(&cmd, cidir, buf);
@@ -295,10 +309,12 @@ maintscript_new(struct pkginfo *pkg, const char *scriptname,
 
 	if (stat(cidir, &stab)) {
 		command_destroy(&cmd);
+
 		if (errno == ENOENT) {
 			debug(dbg_scripts,
 			      "maintscript_new nonexistent %s '%s'",
 			      scriptname, cidir);
+			free(buf);
 			return 0;
 		}
 		ohshite(_("unable to stat %s '%.250s'"), buf, cidir);
@@ -306,6 +322,7 @@ maintscript_new(struct pkginfo *pkg, const char *scriptname,
 	maintscript_exec(pkg, &pkg->available, &cmd, &stab, 0);
 
 	command_destroy(&cmd);
+	free(buf);
 	post_script_tasks();
 
 	return 1;
@@ -320,10 +337,11 @@ maintscript_fallback(struct pkginfo *pkg,
 	struct command cmd;
 	const char *oldscriptpath;
 	struct stat stab;
-	char buf[100];
+	char *buf;
 
 	oldscriptpath = pkg_infodb_get_file(pkg, &pkg->installed, scriptname);
-	sprintf(buf, _("old %s script"), desc);
+	m_asprintf(&buf, _("old %s package %s script"),
+	           pkg_name(pkg, pnaw_nonambig), desc);
 
 	command_init(&cmd, oldscriptpath, buf);
 	command_add_args(&cmd, scriptname, ifok,
@@ -336,6 +354,7 @@ maintscript_fallback(struct pkginfo *pkg,
 			      "maintscript_fallback nonexistent %s '%s'",
 			      scriptname, oldscriptpath);
 			command_destroy(&cmd);
+			free(buf);
 			return 0;
 		}
 		warning(_("unable to stat %s '%.250s': %s"),
@@ -343,6 +362,7 @@ maintscript_fallback(struct pkginfo *pkg,
 	} else {
 		if (!maintscript_exec(pkg, &pkg->installed, &cmd, &stab, SUBPROC_WARN)) {
 			command_destroy(&cmd);
+			free(buf);
 			post_script_tasks();
 			return 1;
 		}
@@ -350,7 +370,8 @@ maintscript_fallback(struct pkginfo *pkg,
 	notice(_("trying script from the new package instead ..."));
 
 	strcpy(cidirrest, scriptname);
-	sprintf(buf, _("new %s script"), desc);
+	m_asprintf(&buf, _("new %s package %s script"),
+	           pkg_name(pkg, pnaw_nonambig), desc);
 
 	command_destroy(&cmd);
 	command_init(&cmd, cidir, buf);
@@ -361,6 +382,7 @@ maintscript_fallback(struct pkginfo *pkg,
 
 	if (stat(cidir, &stab)) {
 		command_destroy(&cmd);
+
 		if (errno == ENOENT)
 			ohshit(_("there is no script in the new version of the package - giving up"));
 		else
@@ -371,6 +393,7 @@ maintscript_fallback(struct pkginfo *pkg,
 	notice(_("... it looks like that went OK"));
 
 	command_destroy(&cmd);
+	free(buf);
 	post_script_tasks();
 
 	return 1;
